/*
 *  Copyright (C) 2003 Ricardo Fernndez Pascual
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "galeon-action-go.h"
#include "galeon-shell.h"
#include "eel-gconf-extensions.h"
#include "prefs-strings.h"
#include "gul-gobject-misc.h"
#include "gul-toolbutton.h"
#include "gul-string.h"
#include "gul-gui.h"

#include <gtk/gtkstock.h>
#include <libgnome/gnome-i18n.h>
#include <string.h>

#define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
#define DEBUG_MSG(x) g_print x
//#define DEBUG_MSG(x) 

static void		galeon_action_go_init			(GaleonActionGo *action);
static void		galeon_action_go_class_init		(GaleonActionGoClass *class);
static void		galeon_action_go_finalize		(GObject *object);
static void		galeon_action_go_connect_proxy		(EggAction *action, GtkWidget *proxy);
static GtkWidget *	galeon_action_go_create_toolitem	(EggAction *action);
static GtkWidget *	galeon_action_go_create_menu_item	(EggAction *action);

static EggActionClass *parent_class = NULL;


MAKE_GET_TYPE (galeon_action_go, "GaleonActionGo", GaleonActionGo,
	       galeon_action_go_class_init, 
	       galeon_action_go_init, GALEON_TYPE_ACTION);


static void
galeon_action_go_class_init (GaleonActionGoClass *class)
{
	GObjectClass *object_class = G_OBJECT_CLASS (class);
	EggActionClass *action_class = EGG_ACTION_CLASS (class);

	object_class->finalize = galeon_action_go_finalize;

	action_class->create_tool_item = galeon_action_go_create_toolitem;
	action_class->create_menu_item = galeon_action_go_create_menu_item;
	action_class->connect_proxy = galeon_action_go_connect_proxy;

	parent_class = g_type_class_peek_parent (class);
}

static void
galeon_action_go_init (GaleonActionGo *action)
{
	g_object_set (G_OBJECT (action), "stock_id", GTK_STOCK_JUMP_TO, NULL);
	g_object_set (G_OBJECT (action), "tooltip", _("Go to the specified location"), NULL);
}

static void
galeon_action_go_finalize (GObject *object)
{
	G_OBJECT_CLASS (parent_class)->finalize (object);
}

static gchar *
galeon_action_go_get_location (GaleonActionGo *b)
{
	GaleonWindow *window;
	char *ret;
	
	window = galeon_action_get_window (GALEON_ACTION (b));
	g_return_val_if_fail (window != NULL, NULL);
	
	ret = galeon_window_get_location_entry_location (window);

	return ret;
}

static void
galeon_action_go_button_clicked_cb (GtkWidget *w, GaleonActionGo *a)
{
	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (a));
	char *location;
	
	g_return_if_fail (embed != NULL);

	location = galeon_action_go_get_location (a);
	g_return_if_fail (location != NULL);
	
	galeon_embed_load_url (embed, location);

	g_free (location);
}


static gboolean
galeon_action_go_button_press_event_cb (GtkWidget *widget, GdkEventButton *event, 
					GaleonActionGo *b)
{
	GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (b));

	if (event->button == 2 && IS_GALEON_WINDOW (window))
	{
		char *location;

		location = galeon_action_go_get_location (b);
		g_return_val_if_fail (location != NULL, FALSE);
	
		if (location)
		{
			GaleonTab *tab = galeon_window_get_active_tab (window);
			galeon_shell_new_tab (galeon_shell, window, tab, location, 
					      GALEON_NEW_TAB_JUMP);
			g_free (location);
		}
		return TRUE;
	}
	
	return FALSE;
}

static void
galeon_action_go_activate_cb (GtkMenuItem *mi, GaleonActionGo *b)
{
	GaleonEmbed *embed = galeon_action_get_embed (GALEON_ACTION (b));
	char *location;
	
	g_return_if_fail (embed != NULL);

	location = galeon_action_go_get_location (b);
	g_return_if_fail (location != NULL);
	
	galeon_embed_load_url (embed, location);

	g_free (location);
}

static void 
galeon_action_go_new_tab_activate_cb (GtkMenuItem *mi, GaleonActionGo *b)
{
	GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (b));
	char *location;

	location = galeon_action_go_get_location (b);
	g_return_if_fail (location != NULL);
	
	if (location)
	{
		GaleonTab *tab = galeon_window_get_active_tab (window);
		galeon_shell_new_tab (galeon_shell, window, tab, location, 
				      GALEON_NEW_TAB_JUMP | GALEON_NEW_TAB_IN_EXISTING_WINDOW);
		g_free (location);
	}
}

static void 
galeon_action_go_new_window_activate_cb (GtkMenuItem *mi, GaleonActionGo *b)
{
	GaleonWindow *window = galeon_action_get_window (GALEON_ACTION (b));
	char *location;

	location = galeon_action_go_get_location (b);
	g_return_if_fail (location != NULL);
	
	if (location)
	{
		GaleonTab *tab = galeon_window_get_active_tab (window);
		galeon_shell_new_tab (galeon_shell, window, tab, location, 
				      GALEON_NEW_TAB_JUMP | GALEON_NEW_TAB_IN_NEW_WINDOW);
		g_free (location);
	}
}


static GtkWidget *
galeon_action_go_create_toolitem (EggAction *a)
{
	GulToolbutton *b = gul_toolbutton_new ();
	GtkMenuShell *ms;

	gul_toolbutton_set_show_arrow (b, FALSE);
	gul_toolbutton_set_enable_menu (b, TRUE);
	egg_tool_item_set_homogeneous (EGG_TOOL_ITEM (b), FALSE);
	
	g_signal_connect (b, "clicked",
			  G_CALLBACK (galeon_action_go_button_clicked_cb), a);
	g_signal_connect (gul_toolbutton_get_button (b), "button-press-event",
			  G_CALLBACK (galeon_action_go_button_press_event_cb), a);

	ms = gul_toolbutton_get_menu (b);
	
	gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Go To The Specified _Location"), 
				     G_CALLBACK (galeon_action_go_activate_cb), a);
	gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Go To The Specified Location in Another _Tab"), 
				     G_CALLBACK (galeon_action_go_new_tab_activate_cb), a);
	gul_gui_append_new_menuitem (GTK_WIDGET (ms), _("Go To The Specified Location in Another _Window"), 
				     G_CALLBACK (galeon_action_go_new_window_activate_cb), a);
	

	return GTK_WIDGET (b);
}

static GtkWidget *
galeon_action_go_create_menu_item (EggAction *a)
{
	GtkWidget *w = gtk_image_menu_item_new ();
	
	g_signal_connect (w, "activate",
			  G_CALLBACK (galeon_action_go_button_clicked_cb), a);
	g_signal_connect (w, "button-press-event",
			  G_CALLBACK (galeon_action_go_button_press_event_cb), a);

	return w;
}

static void
galeon_action_go_connect_proxy (EggAction *a, GtkWidget *proxy)
{
	(* EGG_ACTION_CLASS (parent_class)->connect_proxy) (a, proxy);
}

